@@ -21,7 +21,7 @@ MA 02110-1301 USA
 /*
   nepim - network pipemeter
 
-  $Id: main.c,v 1.63 2006/08/15 13:20:33 evertonm Exp $
+  $Id: main.c,v 1.64 2007/03/16 12:38:01 evertonm Exp $
  */
 
 #include <stdlib.h>
@@ -106,6 +106,7 @@ static void usage(FILE *out, const char *prog_name)
 	  "  -Q             quietly hide debug errors\n"
 	  "  -y tos         ip tos (defaults to %d, tos=-1 means don't change)\n"
 	  "  -K udp-ka-time UDP keepalive timeout (defaults to %ld us)\n"
+	  "                 'udp-ka-time' can't be lower than %ld us\n"
 	  "  -L             supress output of UDP accumulative loss\n"
 	  "\n"
           "client-only options:\n"
@@ -141,7 +142,9 @@ static void usage(FILE *out, const char *prog_name)
 	  "                 socket-ka: -1 = use system's default\n"
 	  "                             0 = off\n"
 	  "                             1 = on\n"
-	  "  -C             enable app-layer TCP keepalives (defaults to off)\n"
+	  "  -C [interval]  enable app-layer TCP keepalives (defaults to off)\n"
+	  "                 if unspecified, 'interval' defaults to %ld us\n"
+	  "                 'interval' can't be lower than %ld us\n"
 	  "  -N nagle       TCP nagle mode (defaults to %d)\n"
 	  "                 nagle: -1 = use system's default\n"
 	  "                         0 = off (enable TCP_NODELAY)\n"
@@ -178,6 +181,7 @@ static void usage(FILE *out, const char *prog_name)
           nepim_global.udp_read_size,
 	  nepim_global.tos,
 	  (long) nepim_global.udp_keepalive_recv_timer,
+	  (long) nepim_global.udp_keepalive_send_delay,
 	  UDP_OVERHEAD,
 	  TCP_OVERHEAD,
           nepim_global.pipes,
@@ -189,6 +193,8 @@ static void usage(FILE *out, const char *prog_name)
           nepim_global.pause_duration,
           nepim_global.max_greetings,
           nepim_global.socket_keepalive,
+	  nepim_global.tcp_keepalive_recv_timer,
+	  nepim_global.tcp_keepalive_send_delay,
           nepim_global.nagle,
           nepim_global.listen_backlog
           );
@@ -334,8 +340,28 @@ int main(int argc, const char *argv[])
     }
 
     if (!strcmp(arg, "-C")) {
-      nepim_global.tcp_keepalive_require = 1; /* true */
+      int j = i + 1;
+      
+      if (j < argc)
+        if (*argv[j] != '-') {
+	  int interval = atoi(argv[j]);
+          ++i;
+
+	  if (interval < nepim_global.tcp_keepalive_send_delay) {
+	    fprintf(stderr,
+		    "%s: app-layer TCP keepalive timeout (%d us) can't be lower than keepalive interval (%ld us)\n",
+		    nepim_global.prog_name,
+		    interval,
+		    nepim_global.tcp_keepalive_send_delay);
+	    exit(1);
+	  }
+
+	  nepim_global.tcp_keepalive_recv_timer = interval;
+        }
+
+      nepim_global.tcp_keepalive_require   = 1; /* true */
       nepim_global.tcp_keepalive_must_send = 1; /* true */
+
       continue;
     }
 
@@ -369,6 +395,15 @@ int main(int argc, const char *argv[])
         exit(1);
       }
       nepim_global.udp_keepalive_recv_timer = atoi(argv[i]);
+
+      if (nepim_global.udp_keepalive_recv_timer < nepim_global.udp_keepalive_send_delay) {
+	fprintf(stderr,
+		"%s: UDP keepalive timeout (%ld us) can't be lower than keepalive interval (%ld us)\n",
+		nepim_global.prog_name,
+		nepim_global.udp_keepalive_recv_timer,
+		nepim_global.udp_keepalive_send_delay);
+	exit(1);
+      }
       continue;
     }
 
